/*
 ==============================================================================================
 File Name		: 3sum_Serial.c
 Author			: Petros Panayi (R) Spring 2016
 Compile and Run: gcc -Wall -Werror -O3 3sum_Serial.c -o 3sum_Serial.out
 [DEBUG Vers.]	: gcc -Wall -Werror -O3 -DDEBUG 3sum_Serial.c -o 3sum_Serial.out
				: ./3sum_Serial.out 
 Comments		: Serial implementation of 2SUM, 3SUM and MaxSum Algorithms for EPL372 Projects
				: https://en.wikipedia.org/wiki/3SUM
				: Check also Algorithms, 4th Edition at http://algs4.cs.princeton.edu/home/
 ===============================================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "support.h"
#ifdef DEBUG
#define MAX 1024
#else
#define MAX 1024*16
#endif
int array[MAX];

void init(){
	int i = 0;
	for(i =0; i < MAX; i++)
		array[i] = 50-(i%100);
#ifdef DEBUG
	printf("\nInit()\n");
	for(i =0; i < MAX; i++)
		printf("%d ", array[i]);
#else
#endif
}
void Serial_2sum(int max){
	int i,j,counter=0;
	for(i = 0; i < max; i++)
		for(j =i+1; j < max; j++)
				if (array[i]+array[j]==0){
					counter ++;
					#ifdef DEBUG
					printf("\nFound:%d+%d=%d", array[i],array[j],array[i]+array[j]);
					#endif
				}
	printf("\nIn %d elements found:%d 2Sum\n", max, counter);
}

void Serial_3sum(int max){
	int i,j,k,counter=0;
	for(i = 0; i < max; i++)
		for(j =i+1; j < max; j++)
			for(k =j+1; k < max; k++)
				if (array[i]+array[j]+array[k]==0){
					counter ++;
					#ifdef DEBUG
					printf("\nFound:%d+%d+%d=%d", array[i],array[j],array[k],array[i]+array[j]+array[k]);
					#endif
				}
	printf("\nIn %d elements found:%d 3Sum\n", max, counter);
}
void Serial_muxSum(int max){
	int i, j,k, seqStart, seqEnd, maxSum = 0;
	for (i = 0; i < max; i++){
		for (j = i; j < max; j++) {
			int thisSum = 0;
			for (k = i; k <= j; k++)
				thisSum += array[k];
			if (thisSum > maxSum) {
				maxSum = thisSum;
				seqStart = i;
				seqEnd = j;
				#ifdef DEBUG
				printf("\nNEW maxSum i:%d j: %d thisSum=%d maxSum=%d",i,j,thisSum,maxSum);
				#endif
			}
		}
	#ifdef DEBUG
	printf("\n i:%d j: %d ",i,j);
	#endif	
	}
	printf("\nIn %d elements MaxSum found:%d from %d to %d\n", max,maxSum,seqStart, seqEnd);
}
int main(int argc, char *argv[]) {
	int i;
	/* Inialization Phase Starts Here */
	startTime();init(); stopTime();	elapsedTime();
	
	for(i = 128; i <= MAX; i*=2){
		startTime();Serial_2sum(i);stopTime();	elapsedTime();
		startTime();Serial_3sum(i);stopTime();	elapsedTime();
		startTime();Serial_muxSum(i);stopTime(); elapsedTime();
	}
	return (0);

}
